# Generated by Django 2.2.16 on 2020-10-27 14:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('assets', '0003_auto_20200917_1120'),
    ]

    operations = [
        migrations.AlterField(
            model_name='asset',
            name='type',
            field=models.CharField(choices=[('unknown', 'Unknown'), ('image', 'Image'), ('svg_image', 'SVG Image'), ('document', 'Document'), ('audio', 'Audio'), ('video', 'Video')], db_index=True, max_length=255),
        ),
    ]
